#pragma once

#include <map>
#include <memory>
#include <vector>

#include "input.h"
#include "camera.h"
#include "scene.h"
#include "texture.h"
#include "fonts.h"
#include "ui.h"

namespace kek {

struct ActiveKeyboardCapture {
	KeyboardCapture id = KEK_INVALID_ID;
	KeyCharCallback callback;
	Callable uncaptureCallback;
};

struct KekData {
	std::map<InputListener, PeriodicCallback> periodicCallbacks;
	std::map<InputListener, KeyCallback> keyCallbacks;
	std::map<InputListener, KeyCharCallback> keyCharCallbacks;
	std::map<InputListener, MouseCallback> mouseCallbacks;
	std::map<InputListener, MouseButtonCallback> mouseButtonCallbacks;
	std::map<KeyBinding, KeyBindingData> keyBindings;

	GLFWwindow *window;
	Shader *shader;

	Camera *activeCamera;
	Scene *activeScene;

	int screenWidth;
	int screenHeight;

	float lastFrameTime;

	std::map<std::string, std::weak_ptr<Texture>> loadedTextures;

	FT_Library freetype;

	std::vector<UIElement *> uiElements;
	Font *uiDefaultFont;
	Shader *uiRectangleShader;
	GLFWCursorMode uiCursorMode;
	ActiveKeyboardCapture activeKeyboardCapture;
};

extern KekData kekData;

}
